#include <assert.h>

#include "framecpp/config.h"

#include "general/fstream.hh"

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Dictionary.hh"
#include "framecpp/Common/PTR_STRUCT.hh"

namespace FrameCPP
{
  namespace Common
  {
    /// \brief The default size for buffered input and output.
    const FrameBufferInterface::buffer_size_type
    FrameBufferInterface::M_BUFFER_SIZE_DEFAULT = 256 * 1024; // 256K

    FrameBufferInterface::
    ~FrameBufferInterface( )
    {
    }

    //===================================================================
    // StreamBase
    //===================================================================

    StreamBase::cntocid_mapping_type StreamBase::m_class_name_to_class_id;

    const bool StreamBase::m_initialized = StreamBase::initialize( );

    StreamBase::
    StreamBase( )
      :	m_ptr_struct_bytes( 0 ),
	m_crc_file( (CheckSum*)NULL ),
	m_crc_frame( (CheckSum*)NULL )
    {
      m_stream_id_to_fsi_id[ FrameSpec::Info::FSI_FR_SH ]
	= FrameSpec::Info::FSI_FR_SH;
      m_stream_id_to_fsi_id[ FrameSpec::Info::FSI_FR_SE ]
	= FrameSpec::Info::FSI_FR_SE;
    }

    StreamBase::
    ~StreamBase( )
    {
    }

    INT_2U StreamBase::
    GetClassId( const std::string& ClassName ) const
    {
      cntocid_mapping_type::const_iterator
	retval = m_class_name_to_class_id.find( ClassName );

      if ( retval != m_class_name_to_class_id.end( ) )
      {
	return retval->second;
      }
      return FrameSpec::Info::FSI_FR_NULL;
    }

    const Dictionary::streamref_type* StreamBase::
    ReferenceStreamRef( ) const
    {
      return
	dynamic_cast< const Dictionary::streamref_type* > 
	( frameSpecInfo( ).
	  FrameObject( FrameSpec::Info::FSI_COMMON_ELEMENTS ) );
    }

    bool StreamBase::
    initialize( )
    {
      //-----------------------------------------------------------------
      // Initialize mapping table
      //-----------------------------------------------------------------
      m_class_name_to_class_id[ "FrSH" ]
	= FrameSpec::Info::FSI_FR_SH;
      m_class_name_to_class_id[ "FrSE" ]
	= FrameSpec::Info::FSI_FR_SE;
      m_class_name_to_class_id[ "FrameH" ]
	= FrameSpec::Info::FSI_FRAME_H;
      m_class_name_to_class_id[ "FrAdcData" ]
	= FrameSpec::Info::FSI_FR_ADC_DATA;
      m_class_name_to_class_id[ "FrDetector" ]
	= FrameSpec::Info::FSI_FR_DETECTOR;
      m_class_name_to_class_id[ "FrEndOfFile" ]
	= FrameSpec::Info::FSI_FR_END_OF_FILE;
      m_class_name_to_class_id[ "FrEndOfFrame" ]
	= FrameSpec::Info::FSI_FR_END_OF_FRAME;
      m_class_name_to_class_id[ "FrEvent" ]
	= FrameSpec::Info::FSI_FR_EVENT;
      m_class_name_to_class_id[ "FrHistory" ]
	= FrameSpec::Info::FSI_FR_HISTORY;
      m_class_name_to_class_id[ "FrMsg" ]
	= FrameSpec::Info::FSI_FR_MSG;
      m_class_name_to_class_id[ "FrProcData" ]
	= FrameSpec::Info::FSI_FR_PROC_DATA;
      m_class_name_to_class_id[ "FrRawData" ]
	= FrameSpec::Info::FSI_FR_RAW_DATA;
      m_class_name_to_class_id[ "FrSerData" ]
	= FrameSpec::Info::FSI_FR_SER_DATA;
      m_class_name_to_class_id[ "FrSimData" ]
	= FrameSpec::Info::FSI_FR_SIM_DATA;
      m_class_name_to_class_id[ "FrSimEvent" ]
	= FrameSpec::Info::FSI_FR_SIM_EVENT;
      m_class_name_to_class_id[ "FrStatData" ]
	= FrameSpec::Info::FSI_FR_STAT_DATA;
      m_class_name_to_class_id[ "FrSummary" ]
	= FrameSpec::Info::FSI_FR_SUMMARY;
      m_class_name_to_class_id[ "FrTable" ]
	= FrameSpec::Info::FSI_FR_TABLE;
      m_class_name_to_class_id[ "FrTOC" ]
	= FrameSpec::Info::FSI_FR_TOC;
      m_class_name_to_class_id[ "FrVect" ]
	= FrameSpec::Info::FSI_FR_VECT;
      //-----------------------------------------------------------------
      // Aliases for earlier versions of the frame specification
      //-----------------------------------------------------------------
      m_class_name_to_class_id[ "FrTrigData" ]
	= FrameSpec::Info::FSI_FR_EVENT;
      return true;
    }

    //===================================================================
    // class IStream::resolver
    //===================================================================

    IStream::resolver::
    resolver( PTR_STRUCT_BASE* PtrStruct )
      : m_ptr_struct( PtrStruct )
    {
    }

    IStream::resolver::
    ~resolver( )
    {
      if ( m_ptr_struct )
      {
	delete m_ptr_struct;
	m_ptr_struct = (PTR_STRUCT_BASE*)NULL;
      }
    }

    //===================================================================
    // class IStream::ptr_reference
    //===================================================================

    IStream::ptr_reference::
    ptr_reference( FrameSpec::Object** Ptr, PTR_STRUCT_BASE* PtrStruct )
      : resolver( PtrStruct ),
	m_object( Ptr )
    {
      assert( m_object );
    }

    IStream::ptr_reference::
    ~ptr_reference( )
    {
      // Nothing to do here
    }

    void IStream::ptr_reference::
    Resolve( IStream& Stream )
    {
      //-----------------------------------------------------------------
      // Get all of the objects that are needed to rebuild list.
      //-----------------------------------------------------------------
      Dictionary& dict( Stream.GetDictionary( ) );

      Dictionary::ptr_struct_key_type key( m_ptr_struct->Class( ),
					   m_ptr_struct->Instance( ) );
      const Dictionary::streamref_type* sr = dict.Ref( key );
      //-----------------------------------------------------------------
      // 
      //-----------------------------------------------------------------
      if ( sr )
      {
	*m_object = const_cast< Dictionary::object_type* >( dict.Ref( sr ) );
	dict.Remove( *m_object );
      }
    }

    bool IStream::ptr_reference::
    operator==( const void* RHS ) const
    {
      return ( m_object == RHS );
    }

    //===================================================================
    // class IStream
    //===================================================================

    void IStream::
    Cleanup( )
    {
      for ( resolver_container_type::const_reverse_iterator
	      cur = m_resolver_container.rbegin( ),
	      last = m_resolver_container.rend( );
	    cur != last;
	    ++cur )
      {
	delete *cur;
      }
      m_resolver_container.erase( m_resolver_container.begin( ),
				  m_resolver_container.end( ) );
      StreamBase::Cleanup( );
    }

    std::auto_ptr< PTR_STRUCT_BASE > IStream::
    Next( FrameSpec::Object* Obj )
    {
      //-----------------------------------------------------------------
      // Read the PTR_STRUCT from the stream
      //-----------------------------------------------------------------
      std::auto_ptr< PTR_STRUCT_BASE > next( ReadPtrStruct( ) );
      if ( m_logNextPtr )
      {
	//---------------------------------------------------------------
	// Catalog the information
	//---------------------------------------------------------------
	m_next_container[ Obj ]
	  = Dictionary::ptr_struct_key_type( next->Class( ),
					     next->Instance( ) );
      }
      return next;
    }

    //-------------------------------------------------------------------
    /// Reads the next group of bytes from the stream as a PTR_STRUCT
    ///   as defined by the active frame specification associated with
    ///   the stream.
    //-------------------------------------------------------------------
    std::auto_ptr< PTR_STRUCT_BASE > IStream::
    ReadPtrStruct( )
    {
      std::auto_ptr< PTR_STRUCT_BASE >	retval;

      retval.reset( dynamic_cast< PTR_STRUCT_BASE* >( m_ptr_struct_object
						      ->Create( *this ) ) );

      return retval;
    }

    /// Method to resolve links
    void IStream::
    Resolve( )
    {
      for ( resolver_container_type::const_reverse_iterator
	      cur = m_resolver_container.rbegin( ),
	      last = m_resolver_container.rend( );
	    cur != last;
	    ++cur )
      {
	(*cur)->Resolve( *this );
	delete *cur;
      }
      m_resolver_container.erase( m_resolver_container.begin( ),
				  m_resolver_container.end( ) );
    }

    IStream& IStream::
    operator>>( FrameSpec::Object** Data )
    {
      PushPtr( Data );

      return *this;
    }

    void IStream::
    pushResolver( resolver* Resolver )
    {
      m_resolver_container.push_front( Resolver );
    }

    //===================================================================
    // 
    //===================================================================
    INT_8U OStream::
    GetTOCOffset( ) const
    {
      return 0;
    }

    void OStream::
    Next( const FrameSpec::Object* Obj,
	  const FrameSpec::Object* NextObj )
    {
      m_dictionary.Next( Obj, NextObj );
    }

    const FrameSpec::Object* OStream::
    Next( const FrameSpec::Object* Obj ) const
    {
      return m_dictionary.Next( Obj );
    }


    void StreamBase::
    Cleanup( )
    {
      m_dictionary.Clear( );
    }

    void StreamBase::
    frameSpecInfo( const FrameSpec::Info* Spec )
    {
      if ( Spec == (const FrameSpec::Info*)NULL )
      {
	throw std::invalid_argument( "FrameCPP::Common::frameSpecInfo:"
				     " NULL pointer passed for"
				     " frame specification" );
      }
      m_frame_spec = Spec;
      m_null_stream_ref.reset( frameSpecInfo( ).
			       FrameObject( FrameSpec::Info::FSI_COMMON_ELEMENTS )
			       ->Clone( ) );
      const PTR_STRUCT_BASE* ptr_struct
	= dynamic_cast< const PTR_STRUCT_BASE * >
	( frameSpecInfo( ).FrameObject( FrameSpec::Info::FSI_PTR_STRUCT ) );
      if ( ptr_struct )
      {
	m_ptr_struct_object.
	  reset( dynamic_cast< PTR_STRUCT_BASE * >
		 ( ptr_struct->Clone( ) ) );
      }
      else
      {
	std::ostringstream	msg;

	msg
	  << "FATAL: No PTR_STRUCT class defined for version "
	  << frameSpecInfo( ).Version( )
	  << ":" << frameSpecInfo( ).VersionMinor( )
	  << " of the frame spec."
	  ;;
	throw std::runtime_error( msg.str( ) );
      }
    }

    //-------------------------------------------------------------------
    /// This method adds an object to the queue of objects to be written.
    /// It preserves the order in which the objects should be written
    ///  to the stream by keeping a local iterator indicating the point
    ///  of insertion.
    ///
    /// \param[in] Obj
    ///     The object to be written.
    /// \param[in] Delete
    ///     A boolean indicating if the object should be deleted once
    ///      written to the stream.
    //-------------------------------------------------------------------
    void OStream::
    Push( const FrameSpec::Object* Obj, bool Delete )
    {
      if ( (Obj )
	   && ( m_dictionary.Ref( Obj )
		== ( Dictionary::streamref_type*)NULL ) )
      {
	m_dictionary.RefCreate( Obj, *this );
	// Ensure a valid value when starting
	if ( m_write_objects.size( ) == 0 )
	{
	  m_write_objects_pos = m_write_objects.begin( );
	}
	// Insert the information into the queue
	m_write_objects_pos =
	  m_write_objects.insert( m_write_objects_pos,
				  object_queue_element( Obj, Delete ) );
	// Advance the iterator to just past the object inserted.
	++m_write_objects_pos;
      }
    }

    const PTR_STRUCT_BASE& OStream::
    NextPtrStruct( const FrameSpec::Object* Cur ) const
    {
      const FrameSpec::Object* next = Next( Cur );
      return Reference( next );
    }

    const PTR_STRUCT_BASE& OStream::
    Reference( const FrameSpec::Object* Obj ) const
    {
      //-----------------------------------------------------------------
      // Retrieve the stream reference
      //-----------------------------------------------------------------
      const Dictionary::streamref_type* sr
	= (const Dictionary::streamref_type*)NULL;
      sr = m_dictionary.Ref( Obj );
      if ( sr == (const Dictionary::streamref_type*)NULL )
      {
	sr = dynamic_cast< const Dictionary::streamref_type *>
	  ( m_null_stream_ref.get( ) );
      }
      //-----------------------------------------------------------------
      // Create a PTR_STRUCT
      //-----------------------------------------------------------------
      m_ptr_struct_object->Class( sr->GetClass( ) );
      m_ptr_struct_object->Instance( sr->GetInstance( ) );
      return *m_ptr_struct_object;
    }

  } // namespace - Common
} // namespace FrameCPP

#include "framecpp/Common/IOStream.icc"

template class FrameCPP::Common::FrameBuffer< General::filebuf >;
