#include "framecpp/config.h"

#include "general/types.hh"

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"

#include "framecpp/Version3/FrameH.hh"
#include "framecpp/Version3/FrSE.hh"
#include "framecpp/Version3/FrSH.hh"
#include "framecpp/Version3/FrEndOfFrame.hh"
#include "framecpp/Version3/PTR_STRUCT.hh"

using namespace FrameCPP::Version_3;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;

#define TRACE_MEMORY 0
#if TRACE_MEMORY
#define MEM_ALLOCATE() std::cerr << "MEMORY: Allocate: " << FrameH::getStaticName() << " " << (void*)this << std::endl;
#define MEM_DELETE() std::cerr << "MEMORY: Delete: " << FrameH::getStaticName() << " " << (void*)this << std::endl;
#else
#define MEM_ALLOCATE()
#define MEM_DELETE()
#endif

#define	LM_DEBUG 0

#if LM_DEBUG
#define	AT() std::cerr << __FILE__ << " " << __LINE__ << std::endl;
#else
#define	AT()
#endif

FrameHStorage::
FrameHStorage( )
{
}

FrameHStorage::
FrameHStorage( const FrameHStorage& Source )
  :   m_data( Source.m_data )
{
}

FrameHStorage::
FrameHStorage( const std::string& name,
	       run_type run,
	       INT_4U frame,
	       const GPSTime& time,
	       INT_2U uLeapS,
	       INT_4S LocalTime,
	       const REAL_8 dt,
	       INT_4U dqual )
  : m_data( name, run, frame, time, uLeapS, LocalTime, dt, dqual )
{
}

//=======================================================================
//
//=======================================================================
FrameH::
FrameH( )
  : Common::FrameH( StructDescription( ) )
{
  MEM_ALLOCATE();
}

FrameH::
FrameH( const FrameH& frame,
	detector_copy_type	Copy )
  : Common::FrameH( StructDescription( ) ),
    FrameHStorage( frame ),
    FrameHRefs( frame, Copy )
{
}

FrameH::
FrameH( const std::string& name,
	run_type run,
	INT_4U frame,
	const GPSTime& time,
	INT_2U uLeapS,
	INT_4S LocalTime,
	const REAL_8 dt,
	INT_4U dqual )
  : Common::FrameH( StructDescription( ) ),
    FrameHStorage( name, run, frame, time, uLeapS, LocalTime, dt, dqual )
{
  MEM_ALLOCATE();
}

FrameH::
FrameH( const FrameHStorage& Source )
  : Common::FrameH( StructDescription( ) ),
    FrameHStorage( Source )
{
}

FrameH::
FrameH( IStream& Stream )
  : Common::FrameH( StructDescription( ) )
{
  m_data( Stream );
  m_refs( Stream );
}

FrameH::
~FrameH( )
{
  MEM_DELETE();
}


FrameCPP::cmn_streamsize_type FrameH::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.Bytes( )
    + m_refs.Bytes( )
    ;
}

FrameH* FrameH::
Create( IStream& Stream ) const
{
  return new FrameH( Stream );
}

const char* const FrameH::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrameH::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( FrameH::StructName( ),
	       FrameCPP::Common::FrameH::s_object_id,
	       "Frame Header Structure" ) );
    dataDescription( ret );
    refDescription( ret );

  }

  return &ret;
}

void FrameH::
Write( OStream& Stream ) const
{
#if WORKING
  //---------------------------------------------------------------------
  // Check to see if per frame checksums are to be calculated
  //---------------------------------------------------------------------
  if ( Stream.IsCalculatingChecksumFrame( ) )
  {
    //-------------------------------------------------------------------
    // Yes ...
    // 1) Reset the buffer
    // 2) Add in the FrBase info
    //-------------------------------------------------------------------
    Stream.SetChecksumMethodFrame( );

    INT_8U	length = GetLength( );
    INT_2U	class_id = GetClassId( );
    INT_4U	instance = GetInstance( );

    Stream.calcChecksumFrame( &length, sizeof( length ) );
    Stream.calcChecksumFrame( &class_id, sizeof( class_id ) );
    Stream.calcChecksumFrame( &instance, sizeof( instance ) );
  }
#endif /* WORKING */

  m_data( Stream );
  m_refs( Stream );
  
  Stream.Push( new FrEndOfFrame( GetRun( ), GetFrame( ) ),
	       true );
}

bool FrameH::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return compare( *this, Obj );
}

std::auto_ptr< FrameSpec::Object > FrameH::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrameH* >( this ) );
  }
  throw
    Unimplemented( "Object* FrameH::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrameH::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}

void FrameH::
readSubset( IStream& Stream, INT_4U ElementMask )
{
  m_refs.readSubset( Stream, ElementMask );
}

