#include "framecpp/Common/Description.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version6/FrameSpec.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/FrTOC.hh"

#include "framecpp/Version6/STRING.hh"

using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::TOCInfo;

namespace FrameCPP
{
  namespace Version_6
  {
    //===================================================================
    // FrTOCData
    //===================================================================
    FrTOCData::
    FrTOCData( IStream& Stream )
    {
      nframe_type	nframe;

      Stream >> m_ULeapS
	     >> nframe;
      if ( nframe )
      {
	m_dataQuality.resize( nframe );
	m_GTimeS.resize( nframe );
	m_GTimeN.resize( nframe );
	m_dt.resize( nframe );
	m_runs.resize( nframe );
	m_frame.resize( nframe );
	m_positionH.resize( nframe );
	m_nFirstADC.resize( nframe );
	m_nFirstSer.resize( nframe );
	m_nFirstTable.resize( nframe );
	m_nFirstMsg.resize( nframe );
	Stream 
	  >> m_dataQuality
	  >> m_GTimeS
	  >> m_GTimeN
	  >> m_dt
	  >> m_runs
	  >> m_frame
	  >> m_positionH
	  >> m_nFirstADC
	  >> m_nFirstSer
	  >> m_nFirstTable
	  >> m_nFirstMsg
	  ;
      }
      nsh_type	nsh;
      Stream >> nsh;
      if ( nsh > 0 )
      {
	m_SHid.resize( nsh );
	m_SHname.resize( nsh );
	Stream
	  >> m_SHid
	  >> m_SHname
	  ;
      }
      ndetector_type	ndetector;
      Stream >> ndetector;
      if ( ndetector > 0 )
      {
	m_nameDetector.resize( ndetector );
	m_positionDetector.resize( ndetector );
	Stream
	  >> m_nameDetector
	  >> m_positionDetector
	  ;
      }
    }

    void FrTOCData::
    Description( Common::Description& Desc )
    {
      Desc( FrSE( "ULeapS", "INT_2S",
		  "From the first FrameH in the file" ) );
      //-----------------------------------------------------------------
      Desc( FrSE( "nFrame", "INT_4U",
		  "Number of frames in the file" ) );
      Desc( FrSE( "dataQuality", "*INT_4U",
		  "Array of integer QA words from each FrameH"
		  " (size of nFrames)" ) );
      Desc( FrSE( "GTimeS", "*INT_4U",
		  "Array of integer GPS frae times (size of nFrames" ) );
      Desc( FrSE( "GTimeN", "*INT_4U",
		  "Array of integer GPS residual nanoseconds for the frame"
		  " (size of nFrame" ) );
      Desc( FrSE( "dt", "*REAL_8",
		  "Array of frame durations in seconds( size of nFrames)" ) );
      Desc( FrSE( "runs", "*INT_4S",
		  "Array of run numbers (size of nFrame)" ) );
      Desc( FrSE( "frame", "*INT_4U",
		  "Array of frame numbers (size of nFrame)" ) );
      Desc( FrSE( "positionH", "*INT_8U",
		  "Array of FrameH positions, in bytes, from the"
		  " bginning of file (size of nFrame)" ) );
      Desc( FrSE( "nFirstADC", "*INT_8U",
		  "Array of first FrADCData positions, in bytes,"
		  " from beginning of file (size of nFrame)" ) );
      Desc( FrSE( "nFirstSer", "*INT_8U",
		  "Array of first FrSerData positions in bytes,"
		  " from beginning of file (size of nFrame)" ) );
      Desc( FrSE( "nFirstTable", "*INT_8U",
		  "Array of first FrTable positions, in bytes,"
		  " from beginning of file (size of nFrame)" ) );
      Desc( FrSE( "nFirstMsg", "*INT_8U",
		  "Array of first FrMsg positions, in bytes,"
		  " from beginning of file (size of nFrame)" ) );
      //-----------------------------------------------------------------
      Desc( FrSE( "nSH", "INT_4U",
		  "Number of FrSH structures in the file" ) );
      Desc( FrSE( "SHid", "*INT_2U",
		  "Array of FrSH IDs (size of nSH)" ) );
      Desc( FrSE( "SHname", "*STRING",
		  "Array of FrSH names (size of nSH)" ) );
      //-----------------------------------------------------------------
      Desc( FrSE( "nDetector", "INT_4U",
		  "Number of distinct types of FrDetector in the file" ) );
      Desc( FrSE( "nameDetector", "*STRING",
		  "Array of FrDetector names (size of nDetector)."
		  " They appear alphabetically" ) );
      Desc( FrSE( "positionDetector", "*INT_8U",
		  "Array of FrDetector positions from the beginning"
		  " of file (size of nDetector)." ) );
      
    }

    void FrTOCData::
    write( OStream& Stream ) const
    {
      Stream << m_ULeapS
	     << nframe_type( m_dataQuality.size( ) )
	     << m_dataQuality
	     << m_GTimeS
	     << m_GTimeN
	     << m_dt
	     << m_runs
	     << m_frame
	     << m_positionH
	     << m_nFirstADC
	     << m_nFirstSer
	     << m_nFirstTable
	     << m_nFirstMsg
	     << nsh_type( m_SHid.size( ) )
	     << m_SHid
	     << m_SHname
	     << ndetector_type( m_nameDetector.size( ) )
	     << m_nameDetector
	     << m_positionDetector
	;
    }

  } // namespace - Version_6
} // namespace - FrameCPP
