#ifndef FrameCPP_VERSION_6_FrTOCData_HH
#define FrameCPP_VERSION_6_FrTOCData_HH

#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Common
  {
    class TOCInfo;
  }

  namespace Version_6
  {
    //===================================================================
    //===================================================================
    class FrTOCData
    {
    public:
      typedef INT_4U	data_quality_type;
      typedef REAL_8	dt_type;
      typedef INT_4U	frame_type;
      typedef INT_4U	gtimen_type;
      typedef INT_4U	gtimes_type;
      typedef STRING	namedetector_type;
      typedef INT_4U	ndetector_type;
      typedef INT_8U	nfirstadc_type;
      typedef INT_8U	nfirstmsg_type;
      typedef INT_8U	nfirstser_type;
      typedef INT_8U	nfirsttable_type;
      typedef INT_4U	nframe_type;
      typedef INT_4U	nsh_type;
      typedef INT_8U	positiondetector_type;
      typedef INT_8U	positionh_type;
      typedef INT_4S	runs_type;
      typedef INT_2U	shid_type;
      typedef STRING	shname_type;
      typedef INT_2S	uleaps_type;

      typedef std::vector< data_quality_type >	data_quality_container_type;
      typedef std::vector< dt_type >		dt_container_type;
      typedef std::vector< frame_type >		frame_container_type;
      typedef std::vector< gtimen_type >	gtimen_container_type;
      typedef std::vector< gtimes_type >	gtimes_container_type;
      typedef std::vector< namedetector_type >	namedetector_container_type;
      typedef std::vector< nfirstadc_type >	nfirstadc_container_type;
      typedef std::vector< nfirstmsg_type >	nfirstmsg_container_type;
      typedef std::vector< nfirstser_type >	nfirstser_container_type;
      typedef std::vector< nfirsttable_type >	nfirsttable_container_type;
      typedef std::vector< positiondetector_type >
      positiondetector_container_type;
      typedef std::vector< positionh_type >	positionh_container_type;
      typedef std::vector< runs_type >		runs_container_type;
      typedef std::vector< shid_type >		shid_container_type;
      typedef std::vector< shname_type >	shname_container_type;

      FrTOCData( );

      Common::FrameSpec::size_type
      Bytes( const Common::StreamBase& Stream ) const;

      const data_quality_container_type&	GetDataQuality( ) const;
      const dt_container_type&			GetDt( ) const;
      const frame_container_type&		GetFrame( ) const;
      const gtimes_container_type&		GetGTimeN( ) const;
      const gtimen_container_type&		GetGTimeS( ) const;
      const nfirstadc_container_type&		GetNFirstADC( ) const;
      const nfirstmsg_container_type&		GetNFirstMsg( ) const;
      const nfirstser_container_type&		GetNFirstSer( ) const;
      const nfirsttable_container_type&		GetNFirstTable( ) const;
      nframe_type				GetNFrame( ) const;
      const positionh_container_type&		GetPositionH( ) const;
      const runs_container_type&		GetRuns( ) const;
      const shid_container_type&		GetSHid( ) const;
      const shname_container_type&		GetSHname( ) const;
      uleaps_type				GetULeapS( ) const;
      
      const namedetector_container_type&	GetNameDetector( ) const;
      const positiondetector_container_type&	GetPositionDetector( ) const;

      bool operator==( const FrTOCData& RHS ) const;

      static void Description( Common::Description& Desc );

    protected:
      uleaps_type			m_ULeapS;
      data_quality_container_type	m_dataQuality;
      gtimes_container_type		m_GTimeS;
      gtimen_container_type		m_GTimeN;
      dt_container_type			m_dt;
      runs_container_type		m_runs;
      frame_container_type		m_frame;
      positiondetector_container_type	m_positionDetector;
      positionh_container_type		m_positionH;
      namedetector_container_type	m_nameDetector;
      nfirstadc_container_type		m_nFirstADC;
      nfirstser_container_type		m_nFirstSer;
      nfirsttable_container_type	m_nFirstTable;
      nfirstmsg_container_type		m_nFirstMsg;
      shid_container_type		m_SHid;
      shname_container_type		m_SHname;

      FrTOCData( Common::IStream& Stream );
      void write( Common::OStream& Stream ) const;

    };

    inline FrTOCData::
    FrTOCData( )
    {
    }

    inline Common::FrameSpec::size_type FrTOCData::
    Bytes( const Common::StreamBase& Stream ) const
    {
      Common::FrameSpec::size_type	retval = 0;

      //-----------------------------------------------------------------
      // main header block
      //-----------------------------------------------------------------
      retval +=
	sizeof( m_ULeapS )
	+ sizeof( nframe_type )
	+ ( m_dataQuality.size( )
	    * ( sizeof( data_quality_type )
		+ sizeof( gtimes_type )
		+ sizeof( gtimen_type )
		+ sizeof( dt_type )
		+ sizeof( runs_type )
		+ sizeof( frame_type )
		+ sizeof( positionh_type )
		+ sizeof( nfirstadc_type )
		+ sizeof( nfirstmsg_type )
		+ sizeof( nfirstser_type )
		+ sizeof( nfirsttable_type )
		) )
	;
      //-----------------------------------------------------------------
      // SH elements
      //-----------------------------------------------------------------
      retval +=
	sizeof( nsh_type )
	+ ( sizeof( shid_type ) * m_SHid.size( ) )
	;
      for( shname_container_type::const_iterator
	     cur = m_SHname.begin( ),
	     last = m_SHname.end( );
	   cur != last;
	   ++cur )
      {
	retval += cur->Bytes( );
      }
      //-----------------------------------------------------------------
      // Detector information
      //-----------------------------------------------------------------
      retval +=
	sizeof( ndetector_type )
	+ ( sizeof( positiondetector_type ) * m_positionDetector.size( ) )
	;
      for ( namedetector_container_type::const_iterator
	      cur = m_nameDetector.begin( ),
	      last = m_nameDetector.end( );
	    cur != last;
	    ++cur )
      {
	retval += cur->Bytes( );
      }
      //-----------------------------------------------------------------
      // Return the results
      //-----------------------------------------------------------------
      return retval;
    }

    inline const FrTOCData::data_quality_container_type& FrTOCData::
    GetDataQuality( ) const
    {
      return m_dataQuality;
    }

    inline const FrTOCData::dt_container_type& FrTOCData::
    GetDt( ) const
    {
      return m_dt;
    }

    inline const FrTOCData::frame_container_type& FrTOCData::
    GetFrame( ) const
    {
      return m_frame;
    }

    inline const FrTOCData::gtimen_container_type& FrTOCData::
    GetGTimeN( ) const
    {
      return m_GTimeN;
    }

    inline const FrTOCData::gtimes_container_type& FrTOCData::
    GetGTimeS( ) const
    {
      return m_GTimeS;
    }

    inline FrTOCData::nframe_type FrTOCData::
    GetNFrame( ) const
    {
      return m_dataQuality.size( );
    }

    inline const FrTOCData::nfirstadc_container_type& FrTOCData::
    GetNFirstADC( ) const
    {
      return m_nFirstADC;
    }

    inline const FrTOCData::nfirstmsg_container_type& FrTOCData::
    GetNFirstMsg( ) const
    {
      return m_nFirstMsg;
    }

    inline const FrTOCData::nfirstser_container_type& FrTOCData::
    GetNFirstSer( ) const
    {
      return m_nFirstSer;
    }

    inline const FrTOCData::nfirsttable_container_type& FrTOCData::
    GetNFirstTable( ) const
    {
      return m_nFirstTable;
    }

    inline const FrTOCData::positionh_container_type& FrTOCData::
    GetPositionH( ) const
    {
      return m_positionH;
    }

    inline const FrTOCData::runs_container_type& FrTOCData::
    GetRuns( ) const
    {
      return m_runs;
    }

    inline const FrTOCData::shid_container_type& FrTOCData::
    GetSHid( ) const
    {
      return m_SHid;
    }

    inline const FrTOCData::shname_container_type& FrTOCData::
    GetSHname( ) const
    {
      return m_SHname;
    }

    inline const FrTOCData::namedetector_container_type& FrTOCData::
    GetNameDetector( ) const
    {
      return m_nameDetector;
    }

    inline const FrTOCData::positiondetector_container_type& FrTOCData::
    GetPositionDetector( ) const
    {
      return m_positionDetector;
    }

    inline FrTOCData::uleaps_type FrTOCData::
    GetULeapS( ) const
    {
      return m_ULeapS;
    }

    inline bool FrTOCData::
    operator==( const FrTOCData& RHS ) const
    {
#define CMP__(X) (X==RHS.X)

      return ( ( &RHS == this )
	       || ( CMP__(m_ULeapS )
		    && CMP__(m_dataQuality)
		    && CMP__(m_GTimeS)
		    && CMP__(m_GTimeN)
		    && CMP__(m_dt)
		    && CMP__(m_runs)
		    && CMP__(m_frame)
		    && CMP__(m_positionH)
		    && CMP__(m_nFirstADC)
		    && CMP__(m_nFirstSer)
		    && CMP__(m_nFirstTable)
		    && CMP__(m_nFirstMsg)
		    && CMP__(m_SHid)
		    && CMP__(m_SHname)
		    && CMP__(m_nameDetector)
		    && CMP__(m_positionDetector)
		    ) )
	;
#undef CMP__
    }

  } // namespace - Version_6
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_6_FrTOCData_HH */
