#ifndef FrameCPP_VERSION_7_FrameH_HH
#define FrameCPP_VERSION_7_FrameH_HH

#include "general/types.hh"

#include "framecpp/Version6/FrameH.hh"

#include "framecpp/Version7/FrameSpec.hh"

#include "framecpp/Common/Description.hh"
#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/FrameH.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version6/FrameH.hh"

#include "framecpp/Version7/GPSTime.hh"
#include "framecpp/Version7/STRING.hh"
#include "framecpp/Version7/PTR_STRUCT.hh"
#include "framecpp/Version7/FrSE.hh"

#include "framecpp/Version7/FrDetector.hh"
#include "framecpp/Version7/FrEvent.hh"
#include "framecpp/Version7/FrHistory.hh"
#include "framecpp/Version7/FrProcData.hh"
#include "framecpp/Version7/FrRawData.hh"
#include "framecpp/Version7/FrSimData.hh"
#include "framecpp/Version7/FrSimEvent.hh"
#include "framecpp/Version7/FrSummary.hh"
#include "framecpp/Version7/FrTable.hh"
#include "framecpp/Version7/FrVect.hh"

namespace FrameCPP
{
  namespace Common
  {
    class Description;
  }

  namespace Version_7
  {
    //===================================================================
    //===================================================================

    typedef Previous::FrameHStorage FrameHStorage;

    //===================================================================
    //===================================================================

    typedef Previous::FrameHRefsT< FrDetector, FrEvent, FrHistory,
				   FrProcData, FrRawData, FrSimData, FrSimEvent,
				   FrSummary, FrTable, FrVect >
    FrameHRefs;

    //===================================================================
    //===================================================================
    class FrameH
      : public Common::FrameH,
        public FrameHStorage,
	public FrameHRefs
    {
    public:
      //: Default constructor
      FrameH( );
      FrameH( const FrameH& frame,
	      detector_copy_type Copy = SHALLOW_COPY_DETECTORS );
      FrameH( const std::string& name,
	      INT_4S run,
	      INT_4U frame,
	      const GPSTime& time,
	      INT_2U uLeapS,
	      const REAL_8 dt,
	      INT_4U dqual = 0 );
      FrameH( const FrameHStorage& Source );
      FrameH( const Previous::FrameH& Source,
	      Common::IStream* Stream );
      ~FrameH( );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

      virtual FrameH* Create( IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      //: Write the structure to the stream
      virtual void Write( OStream& Stream ) const;

      /// \brief comparison operator
      bool operator==( const FrameH& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      //-----------------------------------------------------------------
      // Interface routines
      //-----------------------------------------------------------------
      virtual void readSubset( Common::IStream& Stream,
			       INT_4U ElementMask );

    private:
      using Common::FrameSpec::Object::Create;

      FrameH( IStream& Stream );

      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    };

    inline FrameH::
    FrameH( const Previous::FrameH& Source,
	    Common::IStream* Stream )
      : Common::FrameH( StructDescription( ) ),
	FrameHStorage( Source )
    {
      if ( Stream )
      {
	const INT_2U max_ref = Previous::FrameHRefs::MAX_REF;

	void* rd( &(m_refs.rawData) );
	const void* srd( Source.AddressOfRawData( ) );

	Stream->ReplaceRef( RefType( ), &( Source.RefType( ) ), max_ref );
	Stream->ReplaceRef( RefUser( ), &( Source.RefUser( ) ), max_ref );
	Stream->ReplaceRef( RefDetectSim( ), &( Source.RefDetectSim( ) ), max_ref );
	Stream->ReplaceRef( RefDetectProc( ), &( Source.RefDetectProc( ) ), max_ref );
	Stream->ReplaceRef( RefHistory( ), &( Source.RefHistory( ) ), max_ref );
	Stream->ReplacePtr( (Common::FrameSpec::Object**)(rd),
			    (const Common::FrameSpec::Object**)(srd),
			    max_ref );
	Stream->ReplaceRef( RefProcData( ), &( Source.RefProcData( ) ), max_ref );
	Stream->ReplaceRef( RefSimData( ), &( Source.RefSimData( ) ), max_ref );
	Stream->ReplaceRef( RefEvent( ), &( Source.RefEvent( ) ), max_ref );
	Stream->ReplaceRef( RefSimEvent( ), &( Source.RefSimEvent( ) ), max_ref );
	Stream->ReplaceRef( RefSummaryData( ), &( Source.RefSummaryData( ) ), max_ref );
	Stream->ReplaceRef( RefAuxData( ), &( Source.RefAuxData( ) ), max_ref );
	Stream->ReplaceRef( RefAuxTable( ), &( Source.RefAuxTable( ) ), max_ref );
      }
    }

    inline std::auto_ptr< Common::FrameSpec::Object > FrameH::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrameH, FrameH >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrameH::
    StructName( )
    {
      static const char* const name = "FrameH";
      return name;
    }

    inline bool FrameH::
    operator==( const FrameH& RHS ) const
    {
      return ( ( m_data == RHS.m_data )
	       && ( m_refs == RHS.m_refs ) );
    }



  } // namespace - Version_7
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_7_FrameH_HH */
