#ifndef FRAME_CPP__TEST__FR_STRUCT_HH
#define FRAME_CPP__TEST__FR_STRUCT_HH

#include <memory>

#include "framecpp/Common/FrameSpec.hh"

typedef FrameCPP::Common::FrameSpec::Object FrObject;
typedef FrameCPP::Common::FrameSpec::Info::frame_object_types FrameObjectTypes;

static FrameCPP::Common::IStream* NULL_ISTREAM
= (FrameCPP::Common::IStream*)NULL;

#define CHECK_STRING_4( P, PA, C, CA, N )		\
	Test.Check( C->CA( ).compare( P->PA( ) ) == 0 ) \
	  << Leader << "field: " #N \
	  << std::endl

#define CHECK_STRING( P, C, A, N )	   \
  CHECK_STRING_4( P, A, C, A, N )

#define CHECK_NUMBER_CONST( V, VA, C, N )	  \
	Test.Check( V->VA( ) == C ) \
	  << Leader << "field: " #N \
	  << std::endl

#define CHECK_NUMBER_4( P, PA, C, CA, N )	   \
	Test.Check( C->CA( ) == P->PA( ) ) \
	  << Leader << "field: " #N \
	  << std::endl

#define CHECK_NUMBER( P, C, A, N )	   \
  CHECK_NUMBER_4( P, A, C, A, N )

inline void
AddressCheck( const void* LHS, const void* RHS,
	      bool Sameness, bool Nullness,
	      const std::string& Leader, General::UnitTest& Test )
{
  //---------------------------------------------------------------------
  // Check if the addresses are the same
  //---------------------------------------------------------------------
  Test.Check( ( LHS == RHS ) == Sameness )
    << Leader << " Address Check"
    << std::endl
    ;
  //---------------------------------------------------------------------
      // Check if the addresses is null
  //---------------------------------------------------------------------
  Test.Check( ( LHS == (void*)NULL ) == Nullness )
    << Leader << " Null Check"
    << std::endl
    ;
}

std::auto_ptr< FrObject >
mk_frame_object( int SpecVersion,
		 FrameCPP::Common::FrameSpec::Info::frame_object_types Type );

template< int V >
std::auto_ptr< FrObject >
mk_frame_object( FrameObjectTypes Type );

template< int V >
void verify_downconvert( FrObject* FrameObj,
			 const std::string& Leader,
			 General::UnitTest& Test );

template< int V >
void verify_upconvert( FrObject* FrameObj,
		       const std::string& Leader,
		       General::UnitTest& Test );

template< typename FrameObjPrevType, typename FrameObjCurType >
void Demote( FrameObjCurType* Cur,
	     FrameObjPrevType*& Previous,
	     int PreviousSpecVersion,
	     std::auto_ptr< FrObject >& DemotedObj )
{
  DemotedObj.reset( Cur->DemoteObject( PreviousSpecVersion, Cur,
				       NULL_ISTREAM ) );
  Previous = dynamic_cast< FrameObjPrevType* >( DemotedObj.get( ) );
  if ( DemotedObj.get( ) == Cur )
  {
    DemotedObj.release( );
  }
}

template< typename FrameObjCurType >
void DemoteOutOfExistance( FrObject* FrameObj,
			   int PreviousSpecVersion,
			   const std::string& Leader,
			   General::UnitTest& Test )
{
  FrameObjCurType* obj = dynamic_cast< FrameObjCurType* >( FrameObj );

  if ( obj )
  {
    std::auto_ptr< FrObject >
      previous( obj->DemoteObject( PreviousSpecVersion,
				   obj, NULL_ISTREAM ) );
    AddressCheck( previous.get( ), obj,
		  false, true,
		  Leader, Test );
  }
}

template< typename FrameObjCurType, typename FrameObjPrevType >
std::auto_ptr< FrameObjPrevType >
DemoteToPrevious( FrameObjCurType* Current,
		  int PreviousSpecVersion,
		  const std::string& Leader,
		  General::UnitTest& Test )
{
  std::auto_ptr< FrameObjPrevType >  previous;

  if ( Current )
  {
    std::auto_ptr< FrObject >
      previous_object( Current->DemoteObject( PreviousSpecVersion,
					      Current, NULL_ISTREAM ) );
    previous.reset( dynamic_cast< FrameObjPrevType* >
		    ( previous_object.get( ) ) );
    if ( previous.get( ) )
    {
      previous_object.release( );
    }
    //-------------------------------------------------------------------
    // Address validation
    //-------------------------------------------------------------------
    AddressCheck( previous.get( ), Current,
		  false, false,
		  Leader, Test );
  }
  return previous;
}

template< typename FrameObjCurType, typename FrameObjPrevType >
void DemoteToSame( FrObject* FrameObj,
		   int PreviousSpecVersion,
		   const std::string& Leader,
		   General::UnitTest& Test )
{
  FrameObjCurType* obj = dynamic_cast< FrameObjCurType* >( FrameObj );

  if ( obj )
  {
    std::auto_ptr< FrObject >
      pobj( obj->DemoteObject( PreviousSpecVersion,
			       obj, NULL_ISTREAM ) );
    FrameObjPrevType* previous
      = dynamic_cast< FrameObjPrevType* >( pobj.get( ) );
    AddressCheck( previous, obj,
		  true, false,
		  Leader, Test );
    if ( (void*)pobj.get( ) == obj )
    {
      pobj.release( );
    }
  }
}

template< typename FrameObjPrevType, typename FrameObjCurType >
void Promote( const FrObject* FrameObj,
	      std::auto_ptr< FrameObjCurType >& Promoted,
	      std::auto_ptr< FrameObjPrevType>& Previous,
	      int PreviousSpecVersion,
	      int SpecVersion )
{
  const FrameObjCurType*	typed_obj
    = dynamic_cast< const FrameObjCurType* >( FrameObj );

  Previous.reset( dynamic_cast< FrameObjPrevType* >
		  ( mk_frame_object( PreviousSpecVersion,
				     FrameCPP::Common::FrameSpec::Info::frame_object_types( FrameObj->GetClass( ) ) ).release( ) ) );
  Promoted.reset( dynamic_cast< FrameObjCurType* >
		  ( typed_obj->PromoteObject( SpecVersion,
					      PreviousSpecVersion,
					      Previous.get( ),
					      NULL_ISTREAM ) ) );
}

template< typename FrameObjCurType, typename FrameObjPrevType >
void PromoteToSame( FrObject* FrameObj,
		    int SpecVersion,
		    int PreviousSpecVersion,
		    const std::string& Leader,
		    General::UnitTest& Test )
{
  using FrameCPP::Common::FrameSpec;

  //---------------------------------------------------------------------
  // 
  //---------------------------------------------------------------------
  FrameObjCurType* fr_object =
    dynamic_cast< FrameObjCurType* >( FrameObj );

  //---------------------------------------------------------------------
  //---------------------------------------------------------------------
  const FrameSpec::Info::frame_object_types
    class_id( FrameSpec::Info::frame_object_types( FrameObj->GetClass( ) ) );

  std::auto_ptr< FrameObjPrevType >
    previous( dynamic_cast< FrameObjPrevType* >
	      ( mk_frame_object( PreviousSpecVersion,
				 class_id ).release( ) ) );

  std::auto_ptr< FrameObjCurType >
    promoted( dynamic_cast< FrameObjCurType* >
	      ( fr_object->PromoteObject( SpecVersion,
					  PreviousSpecVersion,
					  previous.get( ),
					  NULL_ISTREAM ) ) );
  
  AddressCheck( previous.get( ), promoted.get( ),
		true, false,
		Leader, Test );

  if ( (void*)promoted.get( ) == (void*)previous.get( ) )
  {
    promoted.release( );
  }
}

template< typename FrameObjCurType, typename FrameObjPrevType >
void PromoteFromNothing( FrObject* FrameObj,
			 int SpecVersion,
			 int PreviousSpecVersion,
			 const std::string& Leader,
			 General::UnitTest& Test )
{
  using FrameCPP::Common::FrameSpec;

  //---------------------------------------------------------------------
  //
  //---------------------------------------------------------------------
  FrameObjCurType* fr_object =
    dynamic_cast< FrameObjCurType* >( FrameObj );

  //---------------------------------------------------------------------
  // Create instance of object to promote
  //---------------------------------------------------------------------
  const FrameSpec::Info::frame_object_types
    class_id( FrameSpec::Info::frame_object_types( FrameObj->GetClass( ) ) );

  std::auto_ptr< FrameObjPrevType >
    previous( dynamic_cast< FrameObjPrevType* >
	      ( mk_frame_object( PreviousSpecVersion,
				 class_id ).release( ) ) );

  std::auto_ptr< FrameObjCurType >
    promoted( dynamic_cast< FrameObjCurType* >
	      ( fr_object->PromoteObject( SpecVersion,
					  PreviousSpecVersion,
					  previous.get( ),
					  NULL_ISTREAM ) ) );
			 
  
  AddressCheck( previous.get( ), promoted.get( ),
		true, true,
		Leader, Test );

  if ( (void*)promoted.get( ) == (void*)previous.get( ) )
  {
    promoted.release( );
  }
}

template< typename FrameObjCurType, typename FrameObjPrevType >
void
PromoteFromPrevious( FrObject* FrameObj,
		     int SpecVersion,
		     std::auto_ptr< FrameObjCurType >& Current,
		     int PreviousSpecVersion,
		     std::auto_ptr< FrameObjPrevType >& Previous,
		     const std::string& Leader,
		     General::UnitTest& Test )
{
  if ( FrameObj )
  {
    //---------------------------------------------------------------------
    // Handle the promotion
    //---------------------------------------------------------------------
    Promote( FrameObj, Current,
	     Previous, PreviousSpecVersion,
	     SpecVersion );
  }
  else
  {
    Test.Message( ) << "WARNING: NULL FrameObj sent to PromoteFromPrevious"
		    << std::endl
      ;
  }
			 
  //---------------------------------------------------------------------
  // Address validation
  //---------------------------------------------------------------------
  AddressCheck( Current.get( ), Previous.get( ),
		false, false,
		Leader, Test );
}

#endif /* FRAME_CPP__TEST__FR_STRUCT_HH */
