#include "general/config.h"

#include <signal.h>

#include <list>
#include <sstream>
#include <typeinfo>

#include "general/ErrorLog.hh"
#include "general/TaskThread.hh"
#include "general/ThreadPool.hh"
#include "general/Timeout.hh"

namespace General
{

  //=====================================================================
  //=====================================================================
  TimeoutException::
  TimeoutException( const std::string& msg )
    : std::runtime_error( msg.c_str( ) )
  {
  }

  //=====================================================================
  //=====================================================================
  void
  Timeout( Task* CallerTask, int Seconds )
  {
    bool 	timedout( false );
    std::string	task_name( ( CallerTask )
			   ? typeid( *CallerTask ).name( )
			   : "null_timed_task" );

    if ( CallerTask )
    {
      TaskThread*	slave = ThreadPool::Acquire( );

      if ( slave )
      {
	bool joinable = true;

	slave->AddTask( CallerTask );

	//---------------------------------------------------------------
	// Wait for task to complete
	//---------------------------------------------------------------
	slave->TimedWait( Seconds );
	timedout = slave->TimedOut( );
	joinable = slave->Halt( );

#if 0
	std::cerr << "DEBUG: joinable: " << joinable
		  << " timedout: " << timedout
		  << std::endl;
#endif /* 0 */
	
	if ( joinable )
	{
	  // Return the resouce to the pool
	  ThreadPool::Relinquish( slave );
	} // if joinable
      } // if slave
    } // if CallerTask
    if ( timedout )
    {
      std::ostringstream	msg;

      msg << "Unable to finish " << task_name
	  << " within " << Seconds << " seconds";

      throw General::TimeoutException( msg.str( ) );

    }
  } // Timeout()
}
